@@ -1,5 +1,5 @@
 <?php
-/** @var OC_L10N $l */
+/** @var \OCP\IL10N $l */
 /** @var array $_ */
 script('federatedfilesharing', 'settings-admin');
 ?>
@@ -25,4 +25,19 @@ script('federatedfilesharing', 'settings-admin');
 			<?php p($l->t('Allow users on this server to receive shares from other servers'));?>
 		</label><br/>
 	</p>
+	<p>
+		<input type="checkbox" name="lookupServerEnabled" id="lookupServerEnabled" class="checkbox"
+			   value="1" <?php if ($_['lookupServerEnabled']) print_unescaped('checked="checked"'); ?> />
+		<label for="lookupServerEnabled">
+			<?php p($l->t('Search global and public address book for users and let local users publish their data'));?>
+		</label><br/>
+	</p>
+	<p>
+		<input type="checkbox" name="lookupServerUploadEnabled" id="lookupServerUploadEnabled" class="checkbox"
+			   value="1" <?php if ($_['lookupServerUploadEnabled']) print_unescaped('checked="checked"'); ?> />
+		<label for="lookupServerUploadEnabled">
+			<?php p($l->t('Allow users to publish their data to a global and public address book'));?>
+		</label><br/>
+	</p>
+
 </div>
