@@ -40,11 +40,15 @@
 
 namespace OCA\User_LDAP;
 
+use OC\HintException;
+use OCA\User_LDAP\Exceptions\ConstraintViolationException;
 use OCA\User_LDAP\User\IUserTools;
 use OCA\User_LDAP\User\Manager;
 use OCA\User_LDAP\User\OfflineUser;
 use OCA\User_LDAP\Mapping\AbstractMapping;
 
+use OC\ServerNotAvailableException;
+
 /**
  * Class Access
  * @package OCA\User_LDAP
@@ -182,27 +186,27 @@ class Access extends LDAPUtility implements IUserTools {
 		$maxResults = $pagingSize > 20 ? $pagingSize : 500;
 		$this->initPagedSearch($filter, array($dn), array($attr), $maxResults, 0);
 		$dn = $this->helper->DNasBaseParameter($dn);
-		$rr = @$this->ldap->read($cr, $dn, $filter, array($attr));
+		$rr = @$this->invokeLDAPMethod('read', $cr, $dn, $filter, array($attr));
 		if(!$this->ldap->isResource($rr)) {
-			if(!empty($attr)) {
+			if ($attr !== '') {
 				//do not throw this message on userExists check, irritates
 				\OCP\Util::writeLog('user_ldap', 'readAttribute failed for DN '.$dn, \OCP\Util::DEBUG);
 			}
 			//in case an error occurs , e.g. object does not exist
 			return false;
 		}
-		if (empty($attr) && ($filter === 'objectclass=*' || $this->ldap->countEntries($cr, $rr) === 1)) {
+		if ($attr === '' && ($filter === 'objectclass=*' || $this->invokeLDAPMethod('countEntries', $cr, $rr) === 1)) {
 			\OCP\Util::writeLog('user_ldap', 'readAttribute: '.$dn.' found', \OCP\Util::DEBUG);
 			return array();
 		}
-		$er = $this->ldap->firstEntry($cr, $rr);
+		$er = $this->invokeLDAPMethod('firstEntry', $cr, $rr);
 		if(!$this->ldap->isResource($er)) {
 			//did not match the filter, return false
 			return false;
 		}
 		//LDAP attributes are not case sensitive
 		$result = \OCP\Util::mb_array_change_key_case(
-				$this->ldap->getAttributes($cr, $er), MB_CASE_LOWER, 'UTF-8');
+			$this->invokeLDAPMethod('getAttributes', $cr, $er), MB_CASE_LOWER, 'UTF-8');
 		$attr = mb_strtolower($attr, 'UTF-8');
 
 		if(isset($result[$attr]) && $result[$attr]['count'] > 0) {
@@ -221,6 +225,32 @@ class Access extends LDAPUtility implements IUserTools {
 		\OCP\Util::writeLog('user_ldap', 'Requested attribute '.$attr.' not found for '.$dn, \OCP\Util::DEBUG);
 		return false;
 	}
+	
+	/**
+	 * Set password for an LDAP user identified by a DN
+	 *
+	 * @param string $userDN the user in question
+	 * @param string $password the new password
+	 * @return bool
+	 * @throws HintException
+	 * @throws \Exception
+	 */
+	public function setPassword($userDN, $password) {
+		if(intval($this->connection->turnOnPasswordChange) !== 1) {
+			throw new \Exception('LDAP password changes are disabled.');
+		}
+		$cr = $this->connection->getConnectionResource();
+		if(!$this->ldap->isResource($cr)) {
+			//LDAP not available
+			\OCP\Util::writeLog('user_ldap', 'LDAP resource not available.', \OCP\Util::DEBUG);
+			return false;
+		}
+		try {
+			return $this->invokeLDAPMethod('modReplace', $cr, $userDN, $password);
+		} catch(ConstraintViolationException $e) {
+			throw new HintException('Password change rejected.', \OC::$server->getL10N('user_ldap')->t('Password change rejected. Hint: ').$e->getMessage(), $e->getCode());
+		}
+	}
 
 	/**
 	 * checks whether the given attributes value is probably a DN
@@ -304,7 +334,6 @@ class Access extends LDAPUtility implements IUserTools {
 	}
 
 	/**
-	public function ocname2dn($name, $isUser) {
 	 * returns the internal ownCloud name for the given LDAP DN of the group, false on DN outside of search DN or failure
 	 * @param string $fdn the dn of the group object
 	 * @param string $ldapName optional, the display name of the object
@@ -423,8 +452,8 @@ class Access extends LDAPUtility implements IUserTools {
 		}
 
 		if($isUser) {
-			$usernameAttribute = $this->connection->ldapExpertUsernameAttr;
-			if(!empty($usernameAttribute)) {
+			$usernameAttribute = strval($this->connection->ldapExpertUsernameAttr);
+			if ($usernameAttribute !== '') {
 				$username = $this->readAttribute($fdn, $usernameAttribute);
 				$username = $username[0];
 			} else {
@@ -806,12 +835,63 @@ class Access extends LDAPUtility implements IUserTools {
 		return $this->count('objectclass=*', $this->connection->ldapBase, array('dn'), $limit, $offset);
 	}
 
+	/**
+	 * Returns the LDAP handler
+	 * @throws \OC\ServerNotAvailableException
+	 */
+
+	/**
+	 * @return mixed
+	 * @throws \OC\ServerNotAvailableException
+	 */
+	private function invokeLDAPMethod() {
+		$arguments = func_get_args();
+		$command = array_shift($arguments);
+		$cr = array_shift($arguments);
+		if (!method_exists($this->ldap, $command)) {
+			return null;
+		}
+		array_unshift($arguments, $cr);
+		// php no longer supports call-time pass-by-reference
+		// thus cannot support controlPagedResultResponse as the third argument
+		// is a reference
+		$doMethod = function () use ($command, &$arguments) {
+			if ($command == 'controlPagedResultResponse') {
+				throw new \InvalidArgumentException('Invoker does not support controlPagedResultResponse, call LDAP Wrapper directly instead.');
+			} else {
+				return call_user_func_array(array($this->ldap, $command), $arguments);
+			}
+		};
+		try {
+			$ret = $doMethod();
+		} catch (ServerNotAvailableException $e) {
+			/* Server connection lost, attempt to reestablish it
+			 * Maybe implement exponential backoff?
+			 * This was enough to get solr indexer working which has large delays between LDAP fetches.
+			 */
+			\OCP\Util::writeLog('user_ldap', "Connection lost on $command, attempting to reestablish.", \OCP\Util::DEBUG);
+			$this->connection->resetConnectionResource();
+			$cr = $this->connection->getConnectionResource();
+
+			if(!$this->ldap->isResource($cr)) {
+				// Seems like we didn't find any resource.
+				\OCP\Util::writeLog('user_ldap', "Could not $command, because resource is missing.", \OCP\Util::DEBUG);
+				throw $e;
+			}
+
+			$arguments[0] = array_pad([], count($arguments[0]), $cr);
+			$ret = $doMethod();
+		}
+		return $ret;
+	}
+
 	/**
 	 * retrieved. Results will according to the order in the array.
 	 * @param int $limit optional, maximum results to be counted
 	 * @param int $offset optional, a starting point
 	 * @return array|false array with the search result as first value and pagedSearchOK as
 	 * second | false if not successful
+	 * @throws \OC\ServerNotAvailableException
 	 */
 	private function executeSearch($filter, $base, &$attr = null, $limit = null, $offset = null) {
 		if(!is_null($attr) && !is_array($attr)) {
@@ -831,13 +911,10 @@ class Access extends LDAPUtility implements IUserTools {
 		$pagedSearchOK = $this->initPagedSearch($filter, $base, $attr, intval($limit), $offset);
 
 		$linkResources = array_pad(array(), count($base), $cr);
-		$sr = $this->ldap->search($linkResources, $base, $filter, $attr);
-		$error = $this->ldap->errno($cr);
+		$sr = $this->invokeLDAPMethod('search', $linkResources, $base, $filter, $attr);
+		// cannot use $cr anymore, might have changed in the previous call!
+		$error = $this->ldap->errno($this->connection->getConnectionResource());
 		if(!is_array($sr) || $error !== 0) {
-			\OCP\Util::writeLog('user_ldap',
-				'Error when searching: '.$this->ldap->error($cr).
-					' code '.$this->ldap->errno($cr),
-				\OCP\Util::ERROR);
 			\OCP\Util::writeLog('user_ldap', 'Attempt for Paging?  '.print_r($pagedSearchOK, true), \OCP\Util::ERROR);
 			return false;
 		}
@@ -870,7 +947,7 @@ class Access extends LDAPUtility implements IUserTools {
 
 			//browsing through prior pages to get the cookie for the new one
 			if($skipHandling) {
-				return;
+				return false;
 			}
 			// if count is bigger, then the server does not support
 			// paged search. Instead, he did a normal search. We set a
@@ -948,11 +1025,10 @@ class Access extends LDAPUtility implements IUserTools {
 	 * @return int
 	 */
 	private function countEntriesInSearchResults($searchResults) {
-		$cr = $this->connection->getConnectionResource();
 		$counter = 0;
 
 		foreach($searchResults as $res) {
-			$count = intval($this->ldap->countEntries($cr, $res));
+			$count = intval($this->invokeLDAPMethod('countEntries', $this->connection->getConnectionResource(), $res));
 			$counter += $count;
 		}
 
@@ -984,7 +1060,6 @@ class Access extends LDAPUtility implements IUserTools {
 		$findings = array();
 		$savedoffset = $offset;
 		do {
-			$continue = false;
 			$search = $this->executeSearch($filter, $base, $attr, $limit, $offset);
 			if($search === false) {
 				return array();
@@ -1003,7 +1078,7 @@ class Access extends LDAPUtility implements IUserTools {
 			}
 
 			foreach($sr as $res) {
-				$findings = array_merge($findings, $this->ldap->getEntries($cr	, $res ));
+				$findings = array_merge($findings, $this->invokeLDAPMethod('getEntries', $cr, $res));
 			}
 
 			$continue = $this->processPagedSearchStatus($sr, $filter, $base, $findings['count'],
@@ -1130,7 +1205,7 @@ class Access extends LDAPUtility implements IUserTools {
 	private function combineFilter($filters, $operator) {
 		$combinedFilter = '('.$operator;
 		foreach($filters as $filter) {
-			if(!empty($filter) && $filter[0] !== '(') {
+			if ($filter !== '' && $filter[0] !== '(') {
 				$filter = '('.$filter.')';
 			}
 			$combinedFilter.=$filter;
@@ -1213,7 +1288,7 @@ class Access extends LDAPUtility implements IUserTools {
 
 		$search = $this->prepareSearchTerm($search);
 		if(!is_array($searchAttributes) || count($searchAttributes) === 0) {
-			if(empty($fallbackAttribute)) {
+			if ($fallbackAttribute === '') {
 				return '';
 			}
 			$filter[] = $fallbackAttribute . '=' . $search;
@@ -1239,8 +1314,12 @@ class Access extends LDAPUtility implements IUserTools {
 
 		$allowEnum = $config->getAppValue('core', 'shareapi_allow_share_dialog_user_enumeration', 'yes');
 
-		$result = empty($term) ? '*' :
-			$allowEnum !== 'no' ? $term . '*' : $term;
+		$result = $term;
+		if ($term === '') {
+			$result = '*';
+		} else if ($allowEnum !== 'no') {
+			$result = $term . '*';
+		}
 		return $result;
 	}
 
@@ -1287,7 +1366,7 @@ class Access extends LDAPUtility implements IUserTools {
 		$filter       = $this->connection->ldapUserFilter;
 		$base         = $this->connection->ldapBaseUsers;
 
-		if($this->connection->ldapUuidUserAttribute === 'auto' && empty($uuidOverride)) {
+		if ($this->connection->ldapUuidUserAttribute === 'auto' && $uuidOverride === '') {
 			// Sacrebleu! The UUID attribute is unknown :( We need first an
 			// existing DN to be able to reliably detect it.
 			$result = $this->search($filter, $base, ['dn'], 1);
@@ -1343,7 +1422,7 @@ class Access extends LDAPUtility implements IUserTools {
 			return true;
 		}
 
-		if(!empty($uuidOverride) && !$force) {
+		if (is_string($uuidOverride) && trim($uuidOverride) !== '' && !$force) {
 			$this->connection->$uuidAttr = $uuidOverride;
 			return true;
 		}
@@ -1386,7 +1465,7 @@ class Access extends LDAPUtility implements IUserTools {
 		if($this->detectUuidAttribute($dn, $isUser)) {
 			$uuid = $this->readAttribute($dn, $this->connection->$uuidAttr);
 			if( !is_array($uuid)
-				&& !empty($uuidOverride)
+				&& $uuidOverride !== ''
 				&& $this->detectUuidAttribute($dn, $isUser, true)) {
 					$uuid = $this->readAttribute($dn,
 												 $this->connection->$uuidAttr);
@@ -1562,7 +1641,7 @@ class Access extends LDAPUtility implements IUserTools {
 	private function abandonPagedSearch() {
 		if($this->connection->hasPagedResultSupport) {
 			$cr = $this->connection->getConnectionResource();
-			$this->ldap->controlPagedResult($cr, 0, false, $this->lastCookie);
+			$this->invokeLDAPMethod('controlPagedResult', $cr, 0, false, $this->lastCookie);
 			$this->getPagedSearchResultState();
 			$this->lastCookie = '';
 			$this->cookies = array();
@@ -1688,7 +1767,7 @@ class Access extends LDAPUtility implements IUserTools {
 				if(!is_null($cookie)) {
 					//since offset = 0, this is a new search. We abandon other searches that might be ongoing.
 					$this->abandonPagedSearch();
-					$pagedSearchOK = $this->ldap->controlPagedResult(
+					$pagedSearchOK = $this->invokeLDAPMethod('controlPagedResult',
 						$this->connection->getConnectionResource(), $limit,
 						false, $cookie);
 					if(!$pagedSearchOK) {
@@ -1716,9 +1795,9 @@ class Access extends LDAPUtility implements IUserTools {
 			// in case someone set it to 0 … use 500, otherwise no results will
 			// be returned.
 			$pageSize = intval($this->connection->ldapPagingSize) > 0 ? intval($this->connection->ldapPagingSize) : 500;
-			$pagedSearchOK = $this->ldap->controlPagedResult(
-				$this->connection->getConnectionResource(), $pageSize, false, ''
-			);
+			$pagedSearchOK = $this->invokeLDAPMethod('controlPagedResult',
+				$this->connection->getConnectionResource(),
+				$pageSize, false, '');
 		}
 
 		return $pagedSearchOK;
