@@ -106,7 +106,7 @@ class Wizard extends LDAPUtility {
 		} else if ($type === 'objects') {
 			$result = $this->access->countObjects($limit);
 		} else {
-			throw new \Exception('internal error: invalid object type', 500);
+			throw new \Exception('Internal error: Invalid object type', 500);
 		}
 
 		return $result;
@@ -220,7 +220,7 @@ class Wizard extends LDAPUtility {
 		}
 
 		$attr = $this->configuration->ldapUserDisplayName;
-		if($attr !== 'displayName' && !empty($attr)) {
+		if ($attr !== '' && $attr !== 'displayName') {
 			// most likely not the default value with upper case N,
 			// verify it still produces a result
 			$count = intval($this->countUsersWithAttribute($attr, true));
@@ -243,7 +243,7 @@ class Wizard extends LDAPUtility {
 			}
 		};
 
-		throw new \Exception(self::$l->t('Could not detect user display name attribute. Please specify it yourself in advanced ldap settings.'));
+		throw new \Exception(self::$l->t('Could not detect user display name attribute. Please specify it yourself in advanced LDAP settings.'));
 	}
 
 	/**
@@ -262,7 +262,7 @@ class Wizard extends LDAPUtility {
 		}
 
 		$attr = $this->configuration->ldapEmailAttribute;
-		if(!empty($attr)) {
+		if ($attr !== '') {
 			$count = intval($this->countUsersWithAttribute($attr, true));
 			if($count > 0) {
 				return false;
@@ -498,7 +498,7 @@ class Wizard extends LDAPUtility {
 			throw new \Exception('Could not connect to LDAP');
 		}
 
-		$obclasses = array('groupOfNames', 'group', 'posixGroup', '*');
+		$obclasses = array('groupOfNames', 'groupOfUniqueNames', 'group', 'posixGroup', '*');
 		$this->determineFeature($obclasses,
 								'objectclass',
 								'ldap_groupfilter_objectclass',
@@ -552,7 +552,7 @@ class Wizard extends LDAPUtility {
 		}
 		//make sure the use display name is set
 		$displayName = $this->configuration->ldapGroupDisplayName;
-		if(empty($displayName)) {
+		if ($displayName === '') {
 			$d = $this->configuration->getDefaults();
 			$this->applyFind('ldap_group_display_name',
 							 $d['ldap_group_display_name']);
@@ -576,7 +576,7 @@ class Wizard extends LDAPUtility {
 		}
 		//make sure the use display name is set
 		$displayName = $this->configuration->ldapUserDisplayName;
-		if(empty($displayName)) {
+		if ($displayName === '') {
 			$d = $this->configuration->getDefaults();
 			$this->applyFind('ldap_display_name', $d['ldap_display_name']);
 		}
@@ -724,7 +724,7 @@ class Wizard extends LDAPUtility {
 		//this did not help :(
 		//Let's see whether we can parse the Host URL and convert the domain to
 		//a base DN
-		$helper = new Helper();
+		$helper = new Helper(\OC::$server->getConfig());
 		$domain = $helper->getDomainFromURL($this->configuration->ldapHost);
 		if(!$domain) {
 			return false;
@@ -904,7 +904,7 @@ class Wizard extends LDAPUtility {
 							$er = $this->ldap->firstEntry($cr, $rr);
 							$attrs = $this->ldap->getAttributes($cr, $er);
 							$dn = $this->ldap->getDN($cr, $er);
-							if(empty($dn)) {
+							if ($dn == false || $dn === '') {
 								continue;
 							}
 							$filterPart = '(memberof=' . $dn . ')';
@@ -923,7 +923,7 @@ class Wizard extends LDAPUtility {
 				if($parts > 1) {
 					$filter = '(&' . $filter . ')';
 				}
-				if(empty($filter)) {
+				if ($filter === '') {
 					$filter = '(objectclass=*)';
 				}
 				break;
@@ -943,7 +943,6 @@ class Wizard extends LDAPUtility {
 				$cns = $this->configuration->ldapGroupFilterGroups;
 				if(is_array($cns) && count($cns) > 0) {
 					$filter .= '(|';
-					$base = $this->configuration->ldapBase[0];
 					foreach($cns as $cn) {
 						$filter .= '(cn=' . $cn . ')';
 					}
@@ -974,7 +973,7 @@ class Wizard extends LDAPUtility {
 						//fallback
 						$attr = 'cn';
 					}
-					if(!empty($attr)) {
+					if ($attr !== '') {
 						$filterUsername = '(' . $attr . $loginpart . ')';
 						$parts++;
 					}
@@ -1099,8 +1098,10 @@ class Wizard extends LDAPUtility {
 		$agent = $this->configuration->ldapAgentName;
 		$pwd = $this->configuration->ldapAgentPassword;
 
-		return ( (!empty($agent) && !empty($pwd))
-		       || (empty($agent) &&  empty($pwd)));
+		return
+			($agent !== '' && $pwd !== '')
+			||  ($agent === '' && $pwd === '')
+		;
 	}
 
 	/**
@@ -1237,7 +1238,7 @@ class Wizard extends LDAPUtility {
 		if(is_array($setFeatures) && !empty($setFeatures)) {
 			//something is already configured? pre-select it.
 			$this->result->addChange($dbkey, $setFeatures);
-		} else if($po && !empty($maxEntryObjC)) {
+		} else if ($po && $maxEntryObjC !== '') {
 			//pre-select objectclass with most result entries
 			$maxEntryObjC = str_replace($p, '', $maxEntryObjC);
 			$this->applyFind($dbkey, $maxEntryObjC);
