@@ -53,9 +53,12 @@ use OC\Files\Storage\Storage;
 use OC\User\User;
 use OCP\Constants;
 use OCP\Files\Cache\ICacheEntry;
+use OCP\Files\EmptyFileNameException;
 use OCP\Files\FileNameTooLongException;
 use OCP\Files\InvalidCharacterInPathException;
+use OCP\Files\InvalidDirectoryException;
 use OCP\Files\InvalidPathException;
+use OCP\Files\Mount\IMountPoint;
 use OCP\Files\NotFoundException;
 use OCP\Files\ReservedWordException;
 use OCP\Files\UnseekableException;
@@ -430,6 +433,7 @@ class View {
 				echo fread($handle, $chunkSize);
 				flush();
 			}
+			fclose($handle);
 			$size = $this->filesize($path);
 			return $size;
 		}
@@ -560,7 +564,7 @@ class View {
 				$mtime = time();
 			}
 			//if native touch fails, we emulate it by changing the mtime in the cache
-			$this->putFileInfo($path, array('mtime' => $mtime));
+			$this->putFileInfo($path, array('mtime' => floor($mtime)));
 		}
 		return true;
 	}
@@ -1145,6 +1149,8 @@ class View {
 				$unlockLater = false;
 				if ($this->lockingEnabled && $operation === 'fopen' && is_resource($result)) {
 					$unlockLater = true;
+					// make sure our unlocking callback will still be called if connection is aborted
+					ignore_user_abort(true);
 					$result = CallbackWrapper::wrap($result, null, null, function () use ($hooks, $path) {
 						if (in_array('write', $hooks)) {
 							$this->unlockFile($path, ILockingProvider::LOCK_EXCLUSIVE);
@@ -1353,18 +1359,10 @@ class View {
 					//add the sizes of other mount points to the folder
 					$extOnly = ($includeMountPoints === 'ext');
 					$mounts = Filesystem::getMountManager()->findIn($path);
-					foreach ($mounts as $mount) {
+					$info->setSubMounts(array_filter($mounts, function(IMountPoint $mount) use ($extOnly) {
 						$subStorage = $mount->getStorage();
-						if ($subStorage) {
-							// exclude shared storage ?
-							if ($extOnly && $subStorage instanceof \OC\Files\Storage\Shared) {
-								continue;
-							}
-							$subCache = $subStorage->getCache('');
-							$rootEntry = $subCache->get('');
-							$info->addSubEntry($rootEntry, $mount->getMountPoint());
-						}
-					}
+						return !($extOnly && $subStorage instanceof \OCA\Files_Sharing\SharedStorage);
+					}));
 				}
 			}
 
@@ -1794,37 +1792,25 @@ class View {
 	 * @throws InvalidPathException
 	 */
 	public function verifyPath($path, $fileName) {
-
-		$l10n = \OC::$server->getL10N('lib');
-
-		// verify empty and dot files
-		$trimmed = trim($fileName);
-		if ($trimmed === '') {
-			throw new InvalidPathException($l10n->t('Empty filename is not allowed'));
-		}
-		if (\OC\Files\Filesystem::isIgnoredDir($trimmed)) {
-			throw new InvalidPathException($l10n->t('Dot files are not allowed'));
-		}
-
-		// verify database - e.g. mysql only 3-byte chars
-		if (preg_match('%(?:
-      \xF0[\x90-\xBF][\x80-\xBF]{2}      # planes 1-3
-    | [\xF1-\xF3][\x80-\xBF]{3}          # planes 4-15
-    | \xF4[\x80-\x8F][\x80-\xBF]{2}      # plane 16
-)%xs', $fileName)) {
-			throw new InvalidPathException($l10n->t('4-byte characters are not supported in file names'));
-		}
-
 		try {
 			/** @type \OCP\Files\Storage $storage */
 			list($storage, $internalPath) = $this->resolvePath($path);
 			$storage->verifyPath($internalPath, $fileName);
 		} catch (ReservedWordException $ex) {
-			throw new InvalidPathException($l10n->t('File name is a reserved word'));
+			$l = \OC::$server->getL10N('lib');
+			throw new InvalidPathException($l->t('File name is a reserved word'));
 		} catch (InvalidCharacterInPathException $ex) {
-			throw new InvalidPathException($l10n->t('File name contains at least one invalid character'));
+			$l = \OC::$server->getL10N('lib');
+			throw new InvalidPathException($l->t('File name contains at least one invalid character'));
 		} catch (FileNameTooLongException $ex) {
-			throw new InvalidPathException($l10n->t('File name is too long'));
+			$l = \OC::$server->getL10N('lib');
+			throw new InvalidPathException($l->t('File name is too long'));
+		} catch (InvalidDirectoryException $ex) {
+			$l = \OC::$server->getL10N('lib');
+			throw new InvalidPathException($l->t('Dot files are not allowed'));
+		} catch (EmptyFileNameException $ex) {
+			$l = \OC::$server->getL10N('lib');
+			throw new InvalidPathException($l->t('Empty filename is not allowed'));
 		}
 	}
 
